@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2014, 2017, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -147,6 +147,17 @@ public:
 		const char*	sep,
 		size_t		len);
 
+	/** Set up the virtual column template for partition table, and points
+	all m_table_parts[]->vc_templ to it.
+	@param[in]      table           MySQL TABLE object
+	@param[in]      ib_table        InnoDB dict_table_t
+	@param[in]      table_name      Table name (db/table_name) */
+	void
+	set_v_templ(
+		TABLE*		table,
+		dict_table_t*	ib_table,
+		const char*	name);
+
 private:
 	/** Disable default constructor. */
 	Ha_innopart_share() {};
@@ -284,6 +295,16 @@ public:
 		TABLE*			altered_table,
 		Alter_inplace_info*	ha_alter_info,
 		bool			commit);
+
+	/** Notify the storage engine that the table structure (.frm) has
+	been updated.
+
+	ha_partition allows inplace operations that also upgrades the engine
+	if it supports partitioning natively. So if this is the case then
+	we will remove the .par file since it is not used with ha_innopart
+	(we use the internal data dictionary instead). */
+	void
+	notify_table_changed();
 	/** @} */
 
 	// TODO: should we implement init_table_handle_for_HANDLER() ?
@@ -419,13 +440,6 @@ public:
 		Partition_helper::ph_position(record);
 	}
 
-	int
-	rnd_pos_by_record(
-		uchar*	record)
-	{
-		return(Partition_helper::ph_rnd_pos_by_record(record));
-	}
-
 	/* TODO: Implement these! */
 	bool
 	check_if_incompatible_data(
@@ -534,6 +548,15 @@ public:
 		return(0);
 	}
 
+	// TODO: not yet supporting FK.
+	int
+	get_cascade_foreign_key_table_list(
+		THD*				thd,
+		List<st_handler_tablename>*	fk_table_list)
+	{
+		return(0);
+	}
+
 	int
 	read_range_next()
 	{
@@ -577,7 +600,7 @@ public:
 
 	uint
 	alter_flags(
-		uint	flags __attribute__((unused))) const
+		uint	flags MY_ATTRIBUTE((unused))) const
 	{
 		return(HA_PARTITION_FUNCTION_SUPPORTED
 		       | HA_FAST_CHANGE_PARTITION);
@@ -678,10 +701,6 @@ private:
 	void
 	free_blob_heap_array();
 
-	/** Clear the blob heap for the given partition */
-	void
-	clear_blob_heap_part(uint part_id);
-
 	/** Changes the active index of a handle.
 	@param[in]	part_id	Use this partition.
 	@param[in]	keynr	Use this index; MAX_KEY means always
@@ -998,20 +1017,15 @@ private:
 		uchar*		ref,
 		const uchar*	record);
 
-	/** Read record by given record (by its PK) from the last used partition.
-	see handler::rnd_pos_by_record().
-	@param[in,out]	record	Record to position.
-	@return	0 or error number. */
+	/** Read row using position using given record to find.
+	Only useful when position is based on primary key
+	@param[in]	record  Current record in MySQL Row Format.
+	@return error number or 0. */
 	int
-	rnd_pos_by_record_in_last_part(
-		uchar*	record)
-	{
-		/* Not much overhead to use default function.
-		This avoids out-of-sync code. */
-		return(handler::rnd_pos_by_record(record));
-	}
+	rnd_pos_by_record(
+		uchar*  record);
 
-	/** Copy a cached MySQL record.
+        /** Copy a cached MySQL record.
 	@param[out]	to_record	Where to copy the MySQL record.
 	@param[in]	from_record	Which record to copy. */
 	void
@@ -1092,6 +1106,12 @@ private:
 		THD*	thd,
 		int	lock_type);
 
+	THR_LOCK_DATA**
+	store_lock(
+		THD*			thd,
+		THR_LOCK_DATA**		to,
+		thr_lock_type		lock_type);
+
 	int
 	write_row(
 		uchar*	record)
@@ -1176,6 +1196,14 @@ private:
 	}
 	/** @} */
 
+	/** Fill in data_dir_path and tablespace name from internal data
+	dictionary.
+	@param	part_elem	Partition element to fill.
+	@param	ib_table	InnoDB table to copy from. */
+	void
+	update_part_elem(
+		partition_element*	part_elem,
+		dict_table_t*		ib_table);
 protected:
 	/* Protected handler:: functions specific for native InnoDB partitioning.
 	@see handler.h @{ */
@@ -1192,9 +1220,12 @@ protected:
 		uchar*	record,
 		uchar*	pos);
 
+#ifdef WL6742
+	/* Removing WL6742 as part of Bug 23046302 */
 	int
 	records(
 		ha_rows*	num_rows);
+#endif
 
 	int
 	index_next(
