@@ -18,15 +18,16 @@
  02110-1301  USA
  */
 
-#include <node.h>
 #include <NdbApi.hpp>
+#include <node.h>
 
-typedef v8::Handle<v8::Value> EncoderReader(const NdbDictionary::Column *, 
-                                            char *, size_t);
+using namespace v8;
 
-typedef v8::Handle<v8::Value> EncoderWriter(const NdbDictionary::Column *, 
-                                            v8::Handle<v8::Value>, 
-                                            char *, size_t);
+typedef Handle<Value> EncoderReader(const NdbDictionary::Column *, 
+                                    char *, size_t);
+
+typedef Handle<Value> EncoderWriter(const NdbDictionary::Column *, 
+                                    Handle<Value>, char *, size_t);
 
 typedef struct {
   EncoderReader * read;
@@ -36,4 +37,5 @@ typedef struct {
 
 const NdbTypeEncoder * getEncoderForColumn(const NdbDictionary::Column *);
 
-
+Handle<Object> getBufferForText(const NdbDictionary::Column *, Handle<String>);
+Handle<String> getTextFromBuffer(const NdbDictionary::Column *, Handle<Object>);
